import os
import json
from methods.check_inputs import CheckMethods


class MyCalculator (CheckMethods):

    def __init__(self) :
        pass

    def add(self):
        chose_1 = CheckMethods("Chose the first number : ")
        x = chose_1._check_int()
        chose_2 = CheckMethods("Chose the second number : ")
        y = chose_2._check_int()
        print (f"\n {x} + {y} = " , x+y)
        return x+y
    

    def substract(self):
        chose_1 = CheckMethods("Chose the first number : ")
        x = chose_1._check_int()
        chose_2 = CheckMethods("Chose the second number : ")
        y = chose_2._check_int()
        print (f"\n {x} - {y} = ", x-y)
        return x-y
    
    def multilpy(self):
        chose_1 = CheckMethods("Chose the first number : ")
        x = chose_1._check_int()
        chose_2 = CheckMethods("Chose the second number : ")
        y = chose_2._check_int()
        print (f"\n {x} x {y} = ", x*y)
        return x*y
    
    def divide(self):

        chose_1 = CheckMethods("Chose the first number : ")
        x = chose_1._check_int()
        chose_2 = CheckMethods("Chose the second number : ")
        y = chose_2._check_zero()
        if y == True:
            y = chose_2._check_int()
            print (f"\n {x} / {y} = ", x/y)
        else :
            print("\nError : Division by 0")
    
    def long_calculation(self):

        print('-----------------------------------------------')
        print(" \n To exit or break. Type 'exit' or 'break' ")
        print('\n-----------------------------------------------')


        while True :
                         
            if os.path.isfile("fct_save.json") and os.access("fct_save.json", os.R_OK):

                with open ("fct_save.json", "r") as f:
                    previous = json.load(f)
        
                print ( previous['previous'], end= '')
                    
                next_box = CheckMethods(' ')
                
                if 'exit' in next_box.number.lower() or 'break' in next_box.number.lower():
                    
                    break
                
                next_box.number = previous['previous']  + next_box.number 

                try:            
                    result = self._check_eval(next_box.number) 

                except :
                    print('\n Error. division by 0 or wrong syntax')
                    break
            
                if result != False :
                
                    results = {"previous" : str(result) }
                    json_save = json.dumps(results)
                    with open ("fct_save.json", "w") as f:
                        f.write(json_save)

                if result == 0 or result == 0.0:

                    results = {"previous" : str(0.0) }
                    json_save = json.dumps(results)
                    with open ("fct_save.json", "w") as f:
                        f.write(json_save)
                        
                elif result == False :
                    
                    print('An unexpected error happenned. Please check your input ')
                    break
                        
            else :
                
                box = input(" PLease enter your long calculation :  ")
                result = self._check_eval(box) 
                
                if result :
                    print(f"The result of {box} = {result}" )
                            
                    print( result )
                    results = {"previous" : str(result) }
                    json_save = json.dumps(results)
                    with open ("fct_save.json", "w") as f:
                        f.write(json_save)
                        
                else:
                    
                    print('An unexpected error happenned. Please check your input ')
                    break
        


            





from sympy import * 


class CheckMethods:

    def __init__(self, the_string):

        self.number = input(the_string)

    def _check_int(self):

        while self.number.isdigit() != True: 

            self.number = input('Please enter positiv integer : ')

        return int(self.number)
        
    def _check_float(self):

        while True:
            try:
                self.number = float(self.number)
                return(self.number)
            except ValueError:
                self.number = input('Please input a number :')

    def _check_zero(self):
        if self.check_int() == 0:
            print ('Error : Division by zero not possible ')
            return False
        else:
            return True

    def _check_eval(self, box):

        valid_operations = ["0","1","2","3","4","5","6","7","8","9","+","-","*","/",".","%"]

        for item in box:
            if item in valid_operations:
                no_E = True
            else :
                print("\nError")
                print('-----------------------------------------------')
                print(" \n To exit or break. Type 'exit' or 'break' ")
                print('\n-----------------------------------------------')
                no_E = False
                break 

        if no_E :
            return eval(box)
        else:
            return False
    
    def _check_sympy_equation (self):


        print('-------------Checking the sympyfication ----------\n ')

                    
        try :

            fct = sympify(self.number)

        except :

            print('You enter an invalid syntax. : type -help for list of function')

            return False
        
        return fct
    





from methods.plots import Plot
from methods.calculus import Calculus

def main_menu(My_calculator, Scientific):

        while True :

            print ('\n ------------- Welcome to the simple calculator  -------------')
            print ('----- 1 : Summe ---------')
            print ('----- 2 : Substract ---------')
            print ('----- 3 : Multiple ---------')
            print ('----- 4 : Divide  ---------')
            print ('----- 5 : Long Calculation ---------')
            print ('----- 6 : Enter Scientific Calculator ---------')
            print ('----- 7 : Exit ---------')


            user = input('------Chose your operation here : ')


            if user == '1':
                My_calculator.add()

            elif user == '2' :
                My_calculator.substract()

            elif user == '3' :
                My_calculator.multilpy()

            elif user == '4' :

                My_calculator.divide()
            
            elif user == '5':
                My_calculator.long_calculation()
                
            elif user == '6':
                Scientific.pick_one()

            elif user == '7' or user.lower() == 'exit':
                break
            elif user == '-help':
                print("valid numbers {1,2,3,4,5,6,7}\n") 
                print("syntax for function : \n- sin(x)\n- cos(x)\n- exp(x)\n- ln(x)\n- ** = ^ ")
                print("\n save method only save your last equation!")
            else : 
                print("Please enter a valid input ")



class PLotUI(Plot):

    def __init__(self):
        pass
    
    def pick_plot(self):
        
        while True : 
            print ('\n ------------- Choose your plot!  -------------')
            print ('\n ------------- To save your equation, type -save and -unsave to erase -------------')
            print ('----- 1 : Linear ---------')
            print ('----- 2 : Sin(x) ---------')
            print ('----- 3 : Cos(x) ---------')
            print ('----- 4 : e^(x) ---------')
            print ('----- 5 : Normal distribution  ---------')
            print ('------6: Enter your own plot! ---------')
            print ('----- 7 : Exit ---------')

            user = input('------Chose your operation here : ')

            if user == '1':
                self.plot_linear_graph()
            elif user == '2':
                self.plot_sin()
            elif user == '3':
                self.plot_cos()
            elif user == '4':
                self.euler()
            elif user == '5':
                self.Normal_distribution()
            elif user =='6':
                saves = input('would you like to save the function ? Y or N : ')
                if saves.lower() == 'y' or saves.lower() == 'yes':
                    self.any_plot(save=True)
                elif saves.lower() == 'n' or saves.lower() == 'no':
                    self.any_plot(save=False)
                else:
                    print('Something unexpected happend. Perhaps you input a non valid caracter.')
                    print('The function equation will not be save ')
                    self.any_plot()
            elif user == '7' or user.lower() == 'exit': 
                break
            elif user == '-help':
                print("valid numbers {1,2,3,4,5,6,7}\n") 
                print("syntax for function : \n- sin(x)\n- cos(x)\n- exp(x)\n- ln(x)\n- ** = ^ ")
                print("\n save method only save your last equation!")
            else:
                print("Please enter a valid number {1,2,3,4,5,6,7,8,9} or -help for more info")

class CalculusUI(Calculus):

    def __init__(self):
        pass

    def pick_calculus(self):
        
        while True : 
            print ('\n ------------- Choose your plot!  -------------')
            print ('\n ------------- To save your equation, type -save and -unsave to erase -------------')
            print ('----- 1 : First Derivative ---------')
            print ('----- 2 : Integral without Bounds  ---------')
            print ('----- 3 : Integral with Bounds ---------')
            print ('----- 4 : Solve Cuadratic equation ---------')
            print ('----- 5 : Solve and Save last solution  ---------')
            print ('----- 6 : Exit ---------')

            user = input('------Chose your operation here : ')

            if user == '1':
                self.derivative()
            elif user == '2':
                self.Integrals()
                
            elif user == '3':
                self.Integrals(definitive=True)
            elif user == '4':
                self.solve_quadratic()
            elif user == '5':
                self.solve_quadratic(saves=True)
            elif user == '6' or user.lower() == 'exit':
                break
            elif user == '-help':
                print("valid numbers {1,2,3,4,5,6,7}\n") 
                print("syntax for function : \n- sin(x)\n- cos(x)\n- exp(x)\n- ln(x)\n- ** = ^ ")
                print("\n save method only save your last equation!")
            else:
                print("Please enter a valid number {1,2,3,4,5,6,7,8,9} or -help for more info")

class ScientificUI(CalculusUI, PLotUI):
            
    def __init__(self):
         pass
    
    def pick_one(self ):

        while True :
                print ('\n ------------- Choose your operation !  -------------')
                print ('----- 1 : Plot equations ---------')
                print ('----- 2 : Calculus ---------')
                print ('----- 3 : Exit ---------')


                user = input('------Chose your operation here : ')

                if user == '1':
                    self.pick_plot()
                    break
                elif user == '2':
                    self.pick_calculus()
                    break
                elif user == '3' or user.lower()== 'exit':
                    break
                elif user == '-help':
                    print("valid numbers {1,2,3,4,5,6,7}\n") 
                    print("syntax for function : \n- sin(x)\n- cos(x)\n- exp(x)\n- ln(x)\n- ** = ^ ")
                    print("\n save method only save your last equation!")
                else : 
                    print('Please input a valid option. For more info enter -help')








from methods.check_inputs import CheckMethods
import numpy as np
from sympy.plotting import plot 
from sympy import * 
import json
import os

class Plot(CheckMethods): #Need to check the plot any with plot errors

    def __init__(self):
        pass

    def linspace(self):
        first_x = CheckMethods("Please choose the min value of x: ")
        x_min = first_x._check_float()
        second_x = CheckMethods("Please choose the max value of x: ")
        x_max = second_x._check_float()
        return x_min, x_max
    
    def plot_linear_graph(self):

        print('You have chose the linear graph : f(x) = ax+b')
        print('Please enter the values\n')

        x_min, x_max = self.linspace()
        slope = CheckMethods("Please choose the first parameter a :  ")
        a = slope._check_float()
        origin = CheckMethods("Please choose the second paramater b : ")
        b = origin._check_float()        
        x = Symbol('x')
        plot(((a*x+b),(x,x_min,x_max )),line_color = 'red' , title = 'Linear ') 

    def plot_sin(self):

        print('You have chose the linear graph : f(x) = a*sin(x+phi)')
        print('Please enter the values\n')

        x_min, x_max = self.linspace()
        slope = CheckMethods("Please choose the first parameter a :  ")
        a = slope._check_float()
        origin = CheckMethods("Please choose the phase phi of your sin : ")
        b = origin._check_float()
        x = Symbol('x')
        plot(((a*sin(x+b)),(x,x_min,x_max )),line_color = 'red' , title = 'Linear ')

    def plot_cos(self):

        print('You have chose the linear graph : f(x) = a*cos(x+phi)')
        print('Please enter the values\n')

        x_min, x_max = self.linspace()
        slope = CheckMethods("Please choose the first parameter a :  ")
        a = slope._check_float()
        origin = CheckMethods("Please choose the phase phi of your cos: ")
        b = origin._check_float()
        x = Symbol('x')
        plot(((a*cos(x+b)),(x,x_min,x_max )),line_color = 'red' , title = 'Linear ') 

    def euler(self):

        print('You have chose the linear graph : f(x) = a*exp((x-c)/b)')
        print('Please enter the values\n')

        x_min, x_max = self.linspace()
        slope = CheckMethods("Please choose the first parameter a :  ")
        a = slope._check_float()
        variance = CheckMethods('Enter your mean c : ')
        c = variance._check_float()
        origin = CheckMethods("Please choose the parameter b  : ")
        if origin._check_zero():
            b = origin._check_float()
        else :
            return('Error, division by 0')
        
        x = Symbol('x')
        plot((a*exp((x-c)/b)),(x,x_min,x_max ),line_color = 'red' , title = 'Linear ') 


    def Normal_distribution(self):

        print('You have chose the linear graph : f(x) = [1/sigma*sqrt(2pi)]*exp(-(x-mean)²/2*sigma²)')
        print('Please enter the values\n')

        x_min, x_max = self.linspace()

        mean = CheckMethods("Please choose the mean :  ")
        a = mean._check_float()

        sigma = CheckMethods("Please choose the Standar deviation : ")
        if sigma._check_zero():
            b = sigma._check_float()
        else :
            return('Error, division by 0')
        x = Symbol('x')
        division = 1/b*np.sqrt(2*np.pi)
        plot(division*exp(-(np.square(x-a)/2*np.square(b))),(x,x_min,x_max ),line_color = 'red' , title = 'Linear ') 

    def any_plot(self, save = False):

        print('------This plot is going to be printed outside the box -----')        

        if save == True:

            while True :

                new_plot = input('Do youn want to continue with the last function ? Y or N : ')

                if new_plot.lower() == 'y' or new_plot.lower()=='yes':
                    yes = True
                    break
                elif new_plot.lower() == 'n' or new_plot.lower()== 'no':
                    yes = False
                    break
                else :
                    print('Please enter a yes or no ')

            if yes == True :

                if os.path.isfile("fct_plot.json") and os.access("fct_plot.json", os.R_OK):

                    with open ("fct_plot.json", "r") as f:
                        the_fct = json.load(f)
                        print(the_fct, the_fct['function'])
                    print(f"The previous function was {the_fct['function']} -------\n")
                    fct = sympify(the_fct['function'])
                    x_min = the_fct['x_min']
                    x_max = the_fct['x_max']

                    x = symbols('x')
            
                    plot(fct,(x,x_min,x_max) ,line_color = 'red')
                
                else:
                    print('Sorry, it seems there was no previous equation before ')


                    fct = input('input your equation :  ')
                    
                    try :

                        the_fct = fct
                        x_min, x_max = self.linspace()
                        fct = sympify(fct)

                    except :

                        print('You enter an invalid syntax. : type -help for list of function')

                        return False
                    
                    x = Symbol('x')
                    dictionary = {"function" : the_fct , "x_min": x_min, "x_max": x_max }
                    json_plot = json.dumps(dictionary)
                    plot(fct,(x,x_min,x_max), line_color = 'red')
                    
                    with open ("fct_plot.json", "w") as f:
                        f.write(json_plot)

            if yes == False:

                print(' -----------The new equation will be save --------- ')


                checking = CheckMethods('Input your equation : ')
                equation = checking._check_sympy_equation()
                
                function = str(equation)
                print( ":---------------------",equation, ":---------------------")
                x_min, x_max = self.linspace()
                dictionary = {"function" : function , "x_min": x_min, "x_max": x_max }
                json_save = json.dumps(dictionary)
                x = symbols('x') 
                try:
                    plot(equation,(x,x_min,x_max), line_color = 'red')

                    status = True 
                    
                    
                except:
                    print('Something went wrong. You must have inputed a wrong function. Type -help to display a list of function ')
                    print('---------------The function will not be save----------')
                if status == True:
                     with open ("fct_plot.json", "w") as f:
                        f.write(json_save)

        else: 
            
            checking = CheckMethods('Input your equation : ')
            equation = checking._check_sympy_equation()
            function = str(equation)
            if equation != False:
                print( ":---------------------",equation, ":---------------------")
                x_min, x_max = self.linspace()
                x = symbols('x') 
                plot(equation,(x,x_min,x_max), line_color = 'red')
                # try :

            #     fct = sympify(fct)

            # except :

            #     print('You enter an invalid syntax. : type -help for list of function')

            #     return False
             
            # plot(fct, line_color = 'red')




from methods.check_inputs import CheckMethods
import json
import os
from sympy import * 

class Calculus(CheckMethods):

    def __init__(self):
        pass

    def derivative(self):

        checking = CheckMethods('Input your equation : ')
        equation = checking._check_sympy_equation()
        x=symbols('x')

        solution = diff (equation,x)

        quick_str_check = str(solution)

        if 'Derivative' in quick_str_check:

            print('There is no analytical solution for this function. ')


        else:
            print(f'''The solution of     d( {equation})     =      {solution}
                 ----------                
                    dx ''')
        
        
    def Integrals(self, definitive = False):


        if definitive == False:

            checking = CheckMethods('Input your equation : ')
            equation = checking._check_sympy_equation()
            x=symbols('x')

            solution = integrate(equation,x)

            quick_str_check = str(solution)

            if 'Integral' in quick_str_check:

                print('\nThere is no analytical solution for this function, or the solution is undefiend. ')
                print('\n ---Maybe you inputed a wrong function type -help to see the correct function inputs ! ---')

            else:
                print(f'The solution of  {equation}  : \n \n----{solution} + C ----')
        else : 

            checking = CheckMethods('Input your equation : ')
            equation = checking._check_sympy_equation()
            lower_bound = CheckMethods('Input the lower bound of the integration :')
            if '-infity' in lower_bound.number :
                Lbound = '-oo'

            if 'infity' in lower_bound.number :
                print("Infinit cannot be a lower bound ")
                return ''
            Lbound = lower_bound._check_float()

            uper_bound = CheckMethods('Input the upper bound of the integration :')
            Lbound = '-oo'
            Ubound = 'oo'
                        
            if '-infity' in uper_bound.number :
                print("- Infinit cannot be a upper bound. \nTry inversing the bounds! ")
                return ''

            if 'infity' in uper_bound.number :
                Ubound = 'oo'

            
            Ubound = uper_bound._check_float()

            x=symbols('x')
            solution = integrate(equation,(x,Lbound,Ubound))

            quick_str_check = str(solution)

            if 'Integral' in quick_str_check:

                print('There is no analytical solution for this function, or the solution is undefiend. ')

            else:

                print(f'The solution of ({Lbound},{Ubound}) S {equation}  : \n \n----  {solution} ----')

    def solve_quadratic(self, saves = False):


        if saves == False :

            print('--- you want to solve -----')
            print('--- a*x²+b*x+c = 0 -----')

            a = CheckMethods('Please chose the first parameter a : ')
            first_a = a._check_float()
            b = CheckMethods(' Please chose the second parameter b : ')
            second_b = b._check_float()
            c = CheckMethods(' Please chose the third parameter c : ')            
            third_c = c._check_float()

            h = [None, ';|;'.join(['equation ', 's', 'solutions', '']).split(';')]
            x = Symbol('x')
            t = []
            for e, s in [ ((first_a*x**2 + second_b*x+third_c),x)]:

                how = [{}, dict(dict=True)]

                res = [solve(e, s, **f) for f in how]

                t.append([e, '|', s, '|'] + [ res[1], '|'])
            print('\n')
            print(TableForm(t, headings=h, alignments="<"))

            return e , s 

        else:

            user_choice = CheckMethods('Do you want to see last solution? Yes/No : ')

            if user_choice.number.lower() == 'yes' or user_choice.number.lower() == 'y':

                if os.path.isfile("roots_save.json") and os.access("roots_save.json", os.R_OK):

                    with open ("roots_save.json", "r") as f:
                        previous = json.load(f)

                    print(f"\n {previous['function']} has this solutions {previous['solution']}")

            elif user_choice.number.lower() == 'no' or user_choice.number.lower() == 'n': 

 
                a = CheckMethods('Please chose the first parameter a : ')
                first_a = a._check_float()
                b = CheckMethods(' Please chose the second parameter b : ')
                second_b = b._check_float()
                c = CheckMethods(' Please chose the third parameter c : ')            
                third_c = c._check_float()

                h = [None, ';|;'.join(['equation ', 's', 'solutions', '']).split(';')]
                x = Symbol('x')
                t = []
                the_save = ''
                dictionary = {}

                for e, s in [ ((first_a*x**2 + second_b*x+third_c),x)]:

                    the_save = sympify(e)
                    dictionary['function'] = str(the_save)
                    
                    how = [{}, dict(dict=True)]

                    res = [solve(e, s, **f) for f in how]
                    dictionary["solution"] = str(res[1])
                    t.append([e, '|', s, '|'] + [ res[1], '|'])
                print("\n")
                print(f"----------Here the dictionary {dictionary}--------")
                print("\n")
                json_sols = json.dumps(dictionary)
                with open ("roots_save.json", "w") as f:
                        f.write(json_sols)
                print('\n')
                print(TableForm(t, headings=h, alignments="<"))

                return e , s
            else:
                print("Sorry something wrong happend. Please Check your input")
                print('For this time the function would not be save ! ')

                a = CheckMethods('Please chose the first parameter a : ')
                first_a = a._check_float()
                b = CheckMethods(' Please chose the second parameter b : ')
                second_b = b._check_float()
                c = CheckMethods(' Please chose the third parameter c : ')            
                third_c = c._check_float()

                h = [None, ';|;'.join(['equation ', 's', 'solutions', '']).split(';')]
                x = Symbol('x')
                t = []
                the_save = ''
                dictionary = {}

                for e, s in [ ((first_a*x**2 + second_b*x+third_c),x)]:

                    the_save = sympify(e)
                    dictionary['function'] = str(the_save)
                    
                    how = [{}, dict(dict=True)]

                    res = [solve(e, s, **f) for f in how]
                    dictionary["solution"] = str(res[1])
                    t.append([e, '|', s, '|'] + [ res[1], '|'])


        
                         
            

                

                

        




from utility.ui import ScientificUI
from utility.ui import main_menu
from methods.calculator import MyCalculator

 
First_Object = MyCalculator()
Second_Object = ScientificUI()

if __name__ == '__main__':

    main_menu(First_Object,Second_Object )
        



from setuptools import setup, find_packages
import codecs
import os

VERSION = '0.0.2'
DESCRIPTION = 'Simple advance calculator '
LONG_DESCRIPTION = 'A package that allows to perfoming simple calculus integrals and derivative, ploting any type of graph as well as solving simple cuadratic equations with the help of Numpy and Simpy packages'

# Setting up
setup(
    name="AdvanceCalculatorProject",
    version=VERSION,
    author="Napain (Simon Torrealba)",
    author_email="storrealba24@gmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['simpy','numpy'],
    keywords=['python', 'calculus', 'ploting', 'solver', 'calculator']

)